@@ -3,17 +3,16 @@
 // LICENSE file.
 
 //go:build !gosnmp_nodebug
-// +build !gosnmp_nodebug
 
 package gosnmp
 
-func (l *Logger) Print(v ...interface{}) {
+func (l *Logger) Print(v ...any) {
 	if l.logger != nil {
 		l.logger.Print(v...)
 	}
 }
 
-func (l *Logger) Printf(format string, v ...interface{}) {
+func (l *Logger) Printf(format string, v ...any) {
 	if l.logger != nil {
 		l.logger.Printf(format, v...)
 	}
