@@ -142,8 +142,8 @@ const rxBufSize = 65535 // max size of IPv4 & IPv6 packet
 // For verbose logging to stdout:
 // gosnmp_logger = NewLogger(log.New(os.Stdout, "", 0))
 type LoggerInterface interface {
-	Print(v ...interface{})
-	Printf(format string, v ...interface{})
+	Print(v ...any)
+	Printf(format string, v ...any)
 }
 
 type Logger struct {
@@ -563,8 +563,9 @@ func (packet *SnmpPacket) marshalSNMPV1TrapHeader() ([]byte, error) {
 	if err != nil {
 		return nil, fmt.Errorf("unable to marshal OID: %w", err)
 	}
-	buf.Write([]byte{byte(ObjectIdentifier), byte(len(oidBytes))})
-	buf.Write(oidBytes)
+	if err = marshalTLV(buf, byte(ObjectIdentifier), oidBytes); err != nil {
+		return nil, err
+	}
 
 	// marshal AgentAddress (ip address)
 	ip := net.ParseIP(packet.AgentAddress)
@@ -706,7 +707,6 @@ func (packet *SnmpPacket) marshalVBL() ([]byte, error) {
 	vblBuf := new(bytes.Buffer)
 	for _, pdu := range packet.Variables {
 		// The copy of the 'for' variable "pdu" can be deleted (Go 1.22+)
-		pdu := pdu //nolint:copyloopvar
 		vb, err := marshalVarbind(&pdu)
 		if err != nil {
 			return nil, err
@@ -727,7 +727,13 @@ func (packet *SnmpPacket) marshalVBL() ([]byte, error) {
 	return result, nil
 }
 
-// marshal a varbind
+// marshalVarbind encodes an SNMP variable binding (varbind) as BER.
+// Returns a Sequence TLV containing the OID and its associated value:
+//
+//	Sequence {
+//	  ObjectIdentifier (pdu.Name)
+//	  <Value TLV>      (pdu.Type + pdu.Value)
+//	}
 func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 	oid, err := marshalObjectIdentifier(pdu.Name)
 	if err != nil {
@@ -739,30 +745,20 @@ func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 	// Marshal the PDU type into the appropriate BER
 	switch pdu.Type {
 	case Null:
-		ltmp, err2 := marshalLength(len(oid))
-		if err2 != nil {
-			return nil, err2
+		if err = marshalTLV(tmpBuf, byte(ObjectIdentifier), oid); err != nil {
+			return nil, err
 		}
-		tmpBuf.Write([]byte{byte(ObjectIdentifier)})
-		tmpBuf.Write(ltmp)
-		tmpBuf.Write(oid)
-		tmpBuf.Write([]byte{byte(Null), byte(EndOfContents)})
+		tmpBuf.WriteByte(byte(Null))
+		tmpBuf.WriteByte(byte(EndOfContents))
 
-		ltmp, err2 = marshalLength(tmpBuf.Len())
-		if err2 != nil {
-			return nil, err2
-		}
-		pduBuf.Write([]byte{byte(Sequence)})
-		pduBuf.Write(ltmp)
-		_, err2 = tmpBuf.WriteTo(pduBuf)
-		if err2 != nil {
-			return nil, err2
+		if err = marshalTLV(pduBuf, byte(Sequence), tmpBuf.Bytes()); err != nil {
+			return nil, err
 		}
 
 	case Integer:
-		// Oid
-		tmpBuf.Write([]byte{byte(ObjectIdentifier), byte(len(oid))})
-		tmpBuf.Write(oid)
+		if err = marshalTLV(tmpBuf, byte(ObjectIdentifier), oid); err != nil {
+			return nil, err
+		}
 
 		// Number
 		var intBytes []byte
@@ -776,18 +772,17 @@ func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 		default:
 			return nil, fmt.Errorf("unable to marshal PDU Integer; not byte or int")
 		}
-		tmpBuf.Write([]byte{byte(Integer), byte(len(intBytes))})
-		tmpBuf.Write(intBytes)
-
-		// Sequence, length of oid + integer, then oid/integer data
-		pduBuf.WriteByte(byte(Sequence))
-		pduBuf.WriteByte(byte(len(oid) + len(intBytes) + 4))
-		pduBuf.Write(tmpBuf.Bytes())
+		if err = marshalTLV(tmpBuf, byte(pdu.Type), intBytes); err != nil {
+			return nil, err
+		}
+		if err = marshalTLV(pduBuf, byte(Sequence), tmpBuf.Bytes()); err != nil {
+			return nil, err
+		}
 
 	case Counter32, Gauge32, TimeTicks, Uinteger32:
-		// Oid
-		tmpBuf.Write([]byte{byte(ObjectIdentifier), byte(len(oid))})
-		tmpBuf.Write(oid)
+		if err = marshalTLV(tmpBuf, byte(ObjectIdentifier), oid); err != nil {
+			return nil, err
+		}
 
 		// Number
 		var intBytes []byte
@@ -803,18 +798,17 @@ func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 		default:
 			return nil, fmt.Errorf("unable to marshal pdu.Type %v; unknown pdu.Value %v[type=%T]", pdu.Type, pdu.Value, pdu.Value)
 		}
-		tmpBuf.Write([]byte{byte(pdu.Type), byte(len(intBytes))})
-		tmpBuf.Write(intBytes)
-
-		// Sequence, length of oid + integer, then oid/integer data
-		pduBuf.WriteByte(byte(Sequence))
-		pduBuf.WriteByte(byte(len(oid) + len(intBytes) + 4))
-		pduBuf.Write(tmpBuf.Bytes())
+		if err = marshalTLV(tmpBuf, byte(pdu.Type), intBytes); err != nil {
+			return nil, err
+		}
+		if err = marshalTLV(pduBuf, byte(Sequence), tmpBuf.Bytes()); err != nil {
+			return nil, err
+		}
 
 	case OctetString, BitString, Opaque:
-		// Oid
-		tmpBuf.Write([]byte{byte(ObjectIdentifier), byte(len(oid))})
-		tmpBuf.Write(oid)
+		if err = marshalTLV(tmpBuf, byte(ObjectIdentifier), oid); err != nil {
+			return nil, err
+		}
 
 		// OctetString
 		var octetStringBytes []byte
@@ -826,62 +820,33 @@ func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 		default:
 			return nil, fmt.Errorf("unable to marshal PDU OctetString; not []byte or string")
 		}
-
-		var length []byte
-		length, err = marshalLength(len(octetStringBytes))
-		if err != nil {
-			return nil, fmt.Errorf("unable to marshal PDU length: %w", err)
+		if err = marshalTLV(tmpBuf, byte(pdu.Type), octetStringBytes); err != nil {
+			return nil, err
 		}
-		tmpBuf.WriteByte(byte(pdu.Type))
-		tmpBuf.Write(length)
-		tmpBuf.Write(octetStringBytes)
-
-		tmpBytes := tmpBuf.Bytes()
-
-		length, err = marshalLength(len(tmpBytes))
-		if err != nil {
-			return nil, fmt.Errorf("unable to marshal PDU data length: %w", err)
+		if err = marshalTLV(pduBuf, byte(Sequence), tmpBuf.Bytes()); err != nil {
+			return nil, err
 		}
-		// Sequence, length of oid + octetstring, then oid/octetstring data
-		pduBuf.WriteByte(byte(Sequence))
-
-		pduBuf.Write(length)
-		pduBuf.Write(tmpBytes)
 
 	case ObjectIdentifier:
-		// Oid
-		tmpBuf.Write([]byte{byte(ObjectIdentifier), byte(len(oid))})
-		tmpBuf.Write(oid)
+		if err = marshalTLV(tmpBuf, byte(ObjectIdentifier), oid); err != nil {
+			return nil, err
+		}
 		value := pdu.Value.(string)
-		oidBytes, err := marshalObjectIdentifier(value)
-		if err != nil {
-			return nil, fmt.Errorf("error marshalling ObjectIdentifier: %w", err)
+		oidBytes, encErr := marshalObjectIdentifier(value)
+		if encErr != nil {
+			return nil, fmt.Errorf("error marshalling ObjectIdentifier: %w", encErr)
 		}
-
-		// Oid data
-		var length []byte
-		length, err = marshalLength(len(oidBytes))
-		if err != nil {
-			return nil, fmt.Errorf("error marshalling ObjectIdentifier length: %w", err)
+		if err = marshalTLV(tmpBuf, byte(pdu.Type), oidBytes); err != nil {
+			return nil, err
 		}
-		tmpBuf.WriteByte(byte(pdu.Type))
-		tmpBuf.Write(length)
-		tmpBuf.Write(oidBytes)
-
-		tmpBytes := tmpBuf.Bytes()
-		length, err = marshalLength(len(tmpBytes))
-		if err != nil {
-			return nil, fmt.Errorf("error marshalling ObjectIdentifier data length: %w", err)
+		if err = marshalTLV(pduBuf, byte(Sequence), tmpBuf.Bytes()); err != nil {
+			return nil, err
 		}
-		// Sequence, length of oid + oid, then oid/oid data
-		pduBuf.WriteByte(byte(Sequence))
-		pduBuf.Write(length)
-		pduBuf.Write(tmpBytes)
 
 	case IPAddress:
-		// Oid
-		tmpBuf.Write([]byte{byte(ObjectIdentifier), byte(len(oid))})
-		tmpBuf.Write(oid)
+		if err = marshalTLV(tmpBuf, byte(ObjectIdentifier), oid); err != nil {
+			return nil, err
+		}
 		// OctetString
 		var ipAddressBytes []byte
 		switch value := pdu.Value.(type) {
@@ -893,15 +858,15 @@ func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 		default:
 			return nil, fmt.Errorf("unable to marshal PDU IPAddress; not []byte or string")
 		}
-		tmpBuf.Write([]byte{byte(IPAddress), byte(len(ipAddressBytes))})
-		tmpBuf.Write(ipAddressBytes)
-		// Sequence, length of oid + octetstring, then oid/octetstring data
-		pduBuf.WriteByte(byte(Sequence))
-		pduBuf.WriteByte(byte(len(oid) + len(ipAddressBytes) + 4))
-		pduBuf.Write(tmpBuf.Bytes())
+		if err = marshalTLV(tmpBuf, byte(pdu.Type), ipAddressBytes); err != nil {
+			return nil, err
+		}
+		if err = marshalTLV(pduBuf, byte(Sequence), tmpBuf.Bytes()); err != nil {
+			return nil, err
+		}
 
 	case OpaqueFloat, OpaqueDouble:
-		converters := map[Asn1BER]func(interface{}) ([]byte, error){
+		converters := map[Asn1BER]func(any) ([]byte, error){
 			OpaqueFloat:  marshalFloat32,
 			OpaqueDouble: marshalFloat64,
 		}
@@ -909,71 +874,57 @@ func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 		intBuf := new(bytes.Buffer)
 		intBuf.WriteByte(byte(AsnExtensionTag))
 		intBuf.WriteByte(byte(pdu.Type))
-		intBytes, err := converters[pdu.Type](pdu.Value)
-		if err != nil {
-			return nil, fmt.Errorf("error converting PDU value type %v to %v: %w", pdu.Value, pdu.Type, err)
+		intBytes, encErr := converters[pdu.Type](pdu.Value)
+		if encErr != nil {
+			return nil, fmt.Errorf("error converting PDU value type %v to %v: %w", pdu.Value, pdu.Type, encErr)
 		}
-		intLength, err := marshalLength(len(intBytes))
-		if err != nil {
-			return nil, fmt.Errorf("error marshalling Float type length: %w", err)
+		intLength, encErr := marshalLength(len(intBytes))
+		if encErr != nil {
+			return nil, fmt.Errorf("error marshalling Float type length: %w", encErr)
 		}
 		intBuf.Write(intLength)
 		intBuf.Write(intBytes)
 
-		opaqueLength, err := marshalLength(len(intBuf.Bytes()))
-		if err != nil {
-			return nil, fmt.Errorf("error marshalling Float type length: %w", err)
+		opaqueLength, encErr := marshalLength(len(intBuf.Bytes()))
+		if encErr != nil {
+			return nil, fmt.Errorf("error marshalling Opaque length: %w", encErr)
+		}
+		if err = marshalTLV(tmpBuf, byte(ObjectIdentifier), oid); err != nil {
+			return nil, err
 		}
-		tmpBuf.Write([]byte{byte(ObjectIdentifier), byte(len(oid))})
-		tmpBuf.Write(oid)
 		tmpBuf.WriteByte(byte(Opaque))
 		tmpBuf.Write(opaqueLength)
 		tmpBuf.Write(intBuf.Bytes())
 
-		length, err := marshalLength(len(tmpBuf.Bytes()))
-		if err != nil {
-			return nil, fmt.Errorf("error marshalling Float type length: %w", err)
+		if err = marshalTLV(pduBuf, byte(Sequence), tmpBuf.Bytes()); err != nil {
+			return nil, err
 		}
 
-		// Sequence, length of oid + oid, then oid/oid data
-		pduBuf.WriteByte(byte(Sequence))
-		pduBuf.Write(length)
-		pduBuf.Write(tmpBuf.Bytes())
-
 	case Counter64:
-		tmpBuf.Write([]byte{byte(ObjectIdentifier), byte(len(oid))})
-		tmpBuf.Write(oid)
-		tmpBuf.WriteByte(byte(pdu.Type))
-		intBytes, err := marshalUint64(pdu.Value)
-		if err != nil {
-			return nil, fmt.Errorf("error marshalling: %w", err)
+		if err = marshalTLV(tmpBuf, byte(ObjectIdentifier), oid); err != nil {
+			return nil, err
 		}
-		tmpBuf.WriteByte(byte(len(intBytes)))
-		tmpBuf.Write(intBytes)
-		tmpBytes := tmpBuf.Bytes()
-		length, err := marshalLength(len(tmpBytes))
-		if err != nil {
-			return nil, fmt.Errorf("error marshalling Float type length: %w", err)
+		intBytes, encErr := marshalUint64(pdu.Value)
+		if encErr != nil {
+			return nil, fmt.Errorf("error marshalling Counter64: %w", encErr)
+		}
+		if err = marshalTLV(tmpBuf, byte(pdu.Type), intBytes); err != nil {
+			return nil, err
+		}
+		if err = marshalTLV(pduBuf, byte(Sequence), tmpBuf.Bytes()); err != nil {
+			return nil, err
 		}
-		// Sequence, length of oid + oid, then oid/oid data
-		pduBuf.WriteByte(byte(Sequence))
-		pduBuf.Write(length)
-		pduBuf.Write(tmpBytes)
 
 	case NoSuchInstance, NoSuchObject, EndOfMibView:
-		tmpBuf.Write([]byte{byte(ObjectIdentifier), byte(len(oid))})
-		tmpBuf.Write(oid)
+		if err = marshalTLV(tmpBuf, byte(ObjectIdentifier), oid); err != nil {
+			return nil, err
+		}
 		tmpBuf.WriteByte(byte(pdu.Type))
 		tmpBuf.WriteByte(byte(EndOfContents))
-		tmpBytes := tmpBuf.Bytes()
-		length, err := marshalLength(len(tmpBytes))
-		if err != nil {
-			return nil, fmt.Errorf("error marshalling Null type data length: %w", err)
+
+		if err = marshalTLV(pduBuf, byte(Sequence), tmpBuf.Bytes()); err != nil {
+			return nil, err
 		}
-		// Sequence, length of oid + oid, then oid/oid data
-		pduBuf.WriteByte(byte(Sequence))
-		pduBuf.Write(length)
-		pduBuf.Write(tmpBytes)
 
 	default:
 		return nil, fmt.Errorf("unable to marshal PDU: unknown BER type %q", pdu.Type)
